#if defined (__AVR_ATtiny4313__)
	#define F_CPU 20000000
#endif
#if defined (__AVR_ATmega32__)
	#define F_CPU 16777216
#endif


#if !defined (__MSW_H__)
#define __MSW_H__

#include <util/delay.h>
#include <avr/io.h>
#include <stdint.h>
#include <stdlib.h>

#define sbi(byte,bit) byte|=(1<<bit)
#define cbi(byte,bit) byte&=~(1<<bit)
#define bit(byte,n) (byte>>n)&1

#define PULSE 4
#define BUZZER 6
#define LCD_BL 1
#define GM_SUPPLY 0
//#define SW 0
//*** LCD:BEGIN ***
#define PORT_LCD_CL PORTB
#define PIN_LCD_CL  PINB
#define DDR_LCD_CL  DDRB

#define PORT_LCD_DT  PORTB
#define PIN_LCD_DT  PINB
#define DDR_LCD_DT  DDRB

#define RS  7
#define EN  6

#define LCD_D4  5
#define LCD_D5  4
#define LCD_D6  3
#define LCD_D7  2


void send_4bits_LCD(int8_t b)
{
	PORT_LCD_CL |= (1 << EN);
	
	PORT_LCD_DT &= ~(1 << LCD_D4);
	PORT_LCD_DT &= ~(1 << LCD_D5);
	PORT_LCD_DT &= ~(1 << LCD_D6);
	PORT_LCD_DT &= ~(1 << LCD_D7);

	if ((b >> 0) & 1) PORT_LCD_DT |= (1 << LCD_D4);
	if ((b >> 1) & 1) PORT_LCD_DT |= (1 << LCD_D5);
	if ((b >> 2) & 1) PORT_LCD_DT |= (1 << LCD_D6);
	if ((b >> 3) & 1) PORT_LCD_DT |= (1 << LCD_D7);

	asm("nop");//required by HD44780
	asm("nop");//required by HD44780

	PORT_LCD_CL &= ~(1 << EN);//confirmation
	_delay_us(10);//delay required by HD44780

}

void send_byte_LCD(int8_t b)
{	
	asm("swap %0" : "+r" (b));
	send_4bits_LCD(b);
	asm("swap %0" : "+r" (b));
	send_4bits_LCD(b);
	_delay_us(50);//delay required by HD44780
}

void instruction_mode()
{
	PORT_LCD_CL &= ~(1 << RS);
}

void data_mode()
{
	PORT_LCD_CL |= (1 << RS);
}

void print_char(char c)
{
	data_mode();
	send_byte_LCD((int8_t) c);
}

inline void init_LCD()
{
	DDR_LCD_DT |= (1 << LCD_D4);//set as out
	DDR_LCD_DT |= (1 << LCD_D5);//set as out
	DDR_LCD_DT |= (1 << LCD_D6);//set as out
	DDR_LCD_DT |= (1 << LCD_D7);//set as out

	DDR_LCD_CL |= (1 << RS);
	DDR_LCD_CL |= (1 << EN);

	PORT_LCD_CL &= ~(1 << EN);
	
	instruction_mode();

	_delay_ms(40);//delay required by HD44780
	send_4bits_LCD(0x03);
	_delay_ms(5);//delay required by HD44780
	send_4bits_LCD(0x03);
	_delay_us(120);//delay required by HD44780
	send_4bits_LCD(0x03);
	_delay_us(120);//delay required by HD44780
	send_4bits_LCD(0x02);//4 bit mode
	_delay_us(120);
	send_byte_LCD(0b00101100);//4bit, 2line, 5x8
	send_byte_LCD(0b00001100);//turns on display and cursor off
	send_byte_LCD(0b00000001);//clear LCD
	send_byte_LCD(0b00000110);//increment
	_delay_ms(2);
}

inline void clr_LCD()
{
	instruction_mode();
	send_byte_LCD(0x01);
	_delay_ms(2);
}

void goto_LCD(int8_t b)
{
	instruction_mode();
	send_byte_LCD(0x80 + b);
}

void goto_I_line_LCD()
{
	instruction_mode();
	send_byte_LCD(0x80 + 0);
}

void goto_II_line_LCD()
{
	instruction_mode();
	send_byte_LCD(0x80 + 0x40);
}

void print_string(char *s)
{
	while(*s) print_char(*(s++));
}

int8_t translate_4bits(int8_t b)
{
	b &= 0x0f;
	if (b <= 9) return ((int8_t) '0') + b;
	return ((int8_t) 'A') + b - 0xa;   
}

void print_byte(int8_t b)
{
	asm("swap %0" : "+r" (b));
	print_char((char) translate_4bits(b));
	asm("swap %0" : "+r" (b));
	print_char((char) translate_4bits(b));
}

void print_2bytes(int16_t i)
{
	for (int8_t j=1; j>=0; j--) print_byte(*(((int8_t*) &i) + j));
}

void print_4bytes(int32_t i)
{
	for (int8_t j=3; j>=0; j--) print_byte(*(((int8_t*) &i) + j));
}

//*** LCD:END ***
#endif
